@@ -47,7 +47,8 @@ class ButterflyGroupChannel(ButterflyListChannel,
         @async
         def create_group():
             if self._handle.group is None:
-                connection.msn_client.address_book.add_group(self._handle.name)
+                name = self._handle.name.encode("utf-8")
+                connection.msn_client.address_book.add_group(name)
         create_group()
 
     def AddMembers(self, contacts, message):
@@ -56,8 +57,8 @@ class ButterflyGroupChannel(ButterflyListChannel,
             for contact_handle_id in contacts:
                 contact_handle = self._conn.handle(telepathy.HANDLE_TYPE_CONTACT,
                             contact_handle_id)
-                logger.info("Adding contact %r to pending group %r" %
-                        (contact_handle, self._handle))
+                logger.info("Adding contact %s to pending group %s" %
+                        (unicode(contact_handle), unicode(self._handle)))
                 if contact_handle_id in self.__pending_remove:
                     self.__pending_remove.remove(contact_handle_id)
                 else:
@@ -67,8 +68,8 @@ class ButterflyGroupChannel(ButterflyListChannel,
             for contact_handle_id in contacts:
                 contact_handle = self._conn.handle(telepathy.HANDLE_TYPE_CONTACT,
                             contact_handle_id)
-                logger.info("Adding contact %r to group %r" %
-                        (contact_handle, self._handle))
+                logger.info("Adding contact %s to group %s" %
+                        (unicode(contact_handle), unicode(self._handle)))
                 contact = contact_handle.contact
                 group = self._handle.group
                 if contact is not None and contact.is_member(papyon.Membership.FORWARD):
@@ -82,8 +83,8 @@ class ButterflyGroupChannel(ButterflyListChannel,
             for contact_handle_id in contacts:
                 contact_handle = self._conn.handle(telepathy.HANDLE_TYPE_CONTACT,
                             contact_handle_id)
-                logger.info("Adding contact %r to pending group %r" %
-                        (contact_handle, self._handle))
+                logger.info("Adding contact %s to pending group %s" %
+                        (unicode(contact_handle), unicode(self._handle)))
                 if contact_handle_id in self.__pending_add:
                     self.__pending_add.remove(contact_handle_id)
                 else:
@@ -93,8 +94,8 @@ class ButterflyGroupChannel(ButterflyListChannel,
             for contact_handle_id in contacts:
                 contact_handle = self._conn.handle(telepathy.HANDLE_TYPE_CONTACT,
                             contact_handle_id)
-                logger.info("Removing contact %r from pending group %r" %
-                        (contact_handle, self._handle))
+                logger.info("Removing contact %s from pending group %s" %
+                        (unicode(contact_handle), unicode(self._handle)))
                 contact = contact_handle.contact
                 group = self._handle.group
                 if contact is not None and contact.is_member(papyon.Membership.FORWARD):
@@ -109,25 +110,27 @@ class ButterflyGroupChannel(ButterflyListChannel,
         ab.delete_group(group)
 
     def _filter_contact(self, contact):
-        for group in contact.groups:
-            if group.name == self._handle.name:
-                return (True, False, False)
+        if contact.is_member(papyon.Membership.FORWARD):
+            for group in contact.groups:
+                if group.name.decode("utf-8") == self._handle.name:
+                    return (True, False, False)
         return (False, False, False)
 
     def on_addressbook_group_added(self, group):
-        if group.name == self._handle.name:
+        if group.name.decode("utf-8") == self._handle.name:
             self.AddMembers(self.__pending_add, None)
             self.__pending_add = []
             self.RemoveMembers(self.__pending_remove, None)
             self.__pending_remove = []
 
     def on_addressbook_group_deleted(self, group):
-        if group.name == self._handle.name:
+        if group.name.decode("utf-8") == self._handle.name:
             self.Closed()
             self._conn.remove_channel(self)
 
     def on_addressbook_group_contact_added(self, group, contact):
-        if group.name == self._handle.name:
+        group_name = group.name.decode("utf-8")
+        if group_name == self._handle.name:
             handle = ButterflyHandleFactory(self._conn_ref(), 'contact',
                     contact.account, contact.network_id)
 
@@ -137,11 +140,12 @@ class ButterflyGroupChannel(ButterflyListChannel,
             self.MembersChanged('', added, (), (), (), 0,
                     telepathy.CHANNEL_GROUP_CHANGE_REASON_NONE)
 
-            logger.debug("Contact %s added to group %s" % (handle.name,
-                    group.name))
+            logger.debug("Contact %s added to group %s" %
+                    (handle.name, group_name))
 
     def on_addressbook_group_contact_deleted(self, group, contact):
-        if group.name == self._handle.name:
+        group_name = group.name.decode("utf-8")
+        if group_name == self._handle.name:
             handle = ButterflyHandleFactory(self._conn_ref(), 'contact',
                     contact.account, contact.network_id)
 
@@ -151,6 +155,6 @@ class ButterflyGroupChannel(ButterflyListChannel,
             self.MembersChanged('', (), removed, (), (), 0,
                     telepathy.CHANNEL_GROUP_CHANGE_REASON_NONE)
 
-            logger.debug("Contact %s removed from group %s" % (handle.name,
-                    group.name))
+            logger.debug("Contact %s removed from group %s" %
+                    (handle.name, group_name))
 
