@@ -28,6 +28,8 @@ from butterfly.util.decorator import async
 from butterfly.handle import ButterflyHandleFactory
 from butterfly.media import ButterflySessionHandler
 
+from telepathy.interfaces import CHANNEL_INTERFACE, CHANNEL_INTERFACE_GROUP
+
 __all__ = ['ButterflyMediaChannel']
 
 logger = logging.getLogger('Butterfly.MediaChannel')
@@ -54,8 +56,30 @@ class ButterflyMediaChannel(
         self._call = call
         self._handle = handle
 
+        self._implement_property_get(CHANNEL_INTERFACE, {
+                'InitiatorHandle': lambda: dbus.UInt32(self._initiator.id),
+                'InitiatorID': lambda: self._initiator.name
+                })
+
+        self._implement_property_get(CHANNEL_INTERFACE_GROUP,
+            {'LocalPendingMembers': lambda: self.GetLocalPendingMembersWithInfo() })
+
+        self._add_immutables({
+                'InitiatorHandle': CHANNEL_INTERFACE,
+                'InitiatorID': CHANNEL_INTERFACE,
+                })
+
+        if CHANNEL_INTERFACE + '.InitiatorHandle' in props:
+            self._initiator = conn.handle(telepathy.HANDLE_TYPE_CONTACT,
+                props[CHANNEL_INTERFACE + '.InitiatorHandle'])
+        elif CHANNEL_INTERFACE + '.InitiatorID' in props:
+            self._initiator = ButterflyHandleFactory(conn, 'contact',
+                id=props[CHANNEL_INTERFACE + '.InitiatorHandle'])
+        else:
+            logger.warning('InitiatorID or InitiatorHandle not set on new channel')
+            self._initiator = None
+
         self._session_handler = ButterflySessionHandler(self._conn, self, call.media_session)
-        self.NewSessionHandler(self._session_handler, self._session_handler.subtype)
 
         flags = (telepathy.CHANNEL_GROUP_FLAG_CAN_REMOVE |
                  telepathy.CHANNEL_GROUP_FLAG_MESSAGE_REMOVE |
@@ -95,7 +119,8 @@ class ButterflyMediaChannel(
 
     def RequestStreamDirection(self, id, direction):
         logger.info("Request stream direction %r %r" % (id, direction))
-        self._session_handler.GetStream(id).direction = direction
+        # FIXME: Need to implement changing the stream direction
+        #self._session_handler.GetStream(id).direction = direction
 
     def RemoveStreams(self, streams):
         logger.info("Remove streams %r" % streams)
@@ -108,7 +133,7 @@ class ButterflyMediaChannel(
         return self._conn.GetSelfHandle()
 
     def GetLocalPendingMembersWithInfo(self):
-        info = []
+        info = dbus.Array([], signature="(uuus)")
         for member in self._local_pending:
             info.append((member, self._handle, 0, ''))
         return info
@@ -188,7 +213,6 @@ class ButterflyMediaChannel(
     def on_stream_state_changed(self, handler, state):
         self.StreamStateChanged(handler.id, state)
 
-    @async
     def __add_initial_participants(self):
         added = []
         local_pending = []
