@@ -46,7 +46,8 @@ class ButterflyConnection(telepathy.server.Connection,
         ButterflyCapabilities,
         ButterflyContacts,
         papyon.event.ClientEventInterface,
-        papyon.event.InviteEventInterface):
+        papyon.event.InviteEventInterface,
+        papyon.event.OfflineMessagesEventInterface):
 
     _mandatory_parameters = {
             'account' : 's',
@@ -101,6 +102,8 @@ class ButterflyConnection(telepathy.server.Connection,
             ButterflyContacts.__init__(self)
             papyon.event.ClientEventInterface.__init__(self, self._msn_client)
             papyon.event.InviteEventInterface.__init__(self, self._msn_client)
+            papyon.event.OfflineMessagesEventInterface.__init__(self, self._msn_client)
+
 
             self.set_self_handle(ButterflyHandleFactory(self, 'self'))
 
@@ -127,9 +130,10 @@ class ButterflyConnection(telepathy.server.Connection,
         return self._handles[handle_type, handle_id]
 
     def Connect(self):
-        logger.info("Connecting")
-        self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NONE_SPECIFIED
-        self._msn_client.login(*self._account)
+        if self._status == telepathy.CONNECTION_STATUS_DISCONNECTED:
+            logger.info("Connecting")
+            self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NONE_SPECIFIED
+            self._msn_client.login(*self._account)
 
     def Disconnect(self):
         logger.info("Disconnecting")
@@ -170,14 +174,20 @@ class ButterflyConnection(telepathy.server.Connection,
             self.add_client_handle(handle, sender)
         return handles
 
-    def _generate_props(self, channel_type, handle, suppress_handler):
-        return {
+    def _generate_props(self, channel_type, handle, suppress_handler, initiator_handle=None):
+        props = {
             telepathy.CHANNEL_INTERFACE + '.ChannelType': channel_type,
             telepathy.CHANNEL_INTERFACE + '.TargetHandle': 0 if handle is None else handle.get_id(),
             telepathy.CHANNEL_INTERFACE + '.TargetHandleType': telepathy.HANDLE_TYPE_NONE if handle is None else handle.get_type(),
             telepathy.CHANNEL_INTERFACE + '.Requested': suppress_handler
             }
 
+        if initiator_handle is not None:
+            props[telepathy.CHANNEL_INTERFACE + '.InitiatorHandle'] = initiator_handle.id
+
+        return props
+
+
     @dbus.service.method(telepathy.CONNECTION, in_signature='suub',
         out_signature='o', async_callbacks=('_success', '_error'))
     def RequestChannel(self, type, handle_type, handle_id, suppress_handler,
@@ -280,7 +290,7 @@ class ButterflyConnection(telepathy.server.Connection,
                 participant.account, participant.network_id)
 
         props = self._generate_props(telepathy.CHANNEL_TYPE_TEXT,
-            handle, False)
+            handle, False, initiator_handle=handle)
         channel = self._channel_manager.channel_for_props(props,
             signal=True, conversation=conversation)
 
@@ -291,7 +301,8 @@ class ButterflyConnection(telepathy.server.Connection,
                 call.peer.network_id)
 
         props = self._generate_props(telepathy.CHANNEL_TYPE_STREAMED_MEDIA,
-                handle, False)
+                handle, False, initiator_handle=handle)
+
         channel = self._channel_manager.channel_for_props(props,
                 signal=True, call=call)
 
@@ -303,10 +314,31 @@ class ButterflyConnection(telepathy.server.Connection,
         handle = ButterflyHandleFactory(self, 'contact', session.peer.account,
                 session.peer.network_id)
         props = self._generate_props(telepathy.CHANNEL_TYPE_STREAMED_MEDIA,
-                handle, False)
+                handle, False, initiator_handle=handle)
         channel = self._channel_manager.channel_for_props(props, signal=True,
                 call=session)
 
+    # papyon.event.OfflineMessagesEventInterface
+    def on_oim_messages_received(self, messages):
+        # We got notified we received some offlines messages so we
+        #are going to fetch them
+        self.msn_client.oim_box.fetch_messages(messages)
+
+    # papyon.event.OfflineMessagesEventInterface
+    def on_oim_messages_fetched(self, messages):
+        for message in messages:
+            # Request butterfly text channel (creation, what happen when it exist)
+            sender = message.sender
+            logger.info('received offline message from %s : %s' % (sender.account, message.text))
+            handle = ButterflyHandleFactory(self, 'contact',
+                    sender.account, sender.network_id)
+            props = self._generate_props(telepathy.CHANNEL_TYPE_TEXT,
+                handle, False)
+            channel = self._channel_manager.channel_for_props(props,
+                signal=True)
+            # Notify it of the message
+            channel.offline_message_received(message)
+
     def _advertise_disconnected(self):
         self._manager.disconnected(self)
 
